package com.badlogic.androidgames.framework.gl;

import javax.microedition.khronos.opengles.GL10;

import android.util.FloatMath;

import com.badlogic.androidgames.framework.impl.GLGraphics;
import com.badlogic.androidgames.framework.math.Vector2;

public class SpriteBatcher {
	final float[] vertivesBuffer;
	int bufferIndex;
	final Vertices vertices;
	int numSprites;

	public SpriteBatcher(GLGraphics glGraphics, int maxSprites) {
		this.vertivesBuffer = new float[maxSprites * 4 * 4];
		this.vertices = new Vertices(glGraphics, maxSprites * 4, maxSprites * 6, false, true);
		this.bufferIndex = 0;
		this.numSprites = 0;

		short[] indices = new short[maxSprites * 6];
		int len = indices.length;
		short j = 0;
		for ( int i=0; i<len; i+=6, j+=4 ) {
			indices[i + 0] = (short)(j + 0);
			indices[i + 1] = (short)(j + 1);
			indices[i + 2] = (short)(j + 2);
			indices[i + 3] = (short)(j + 2);
			indices[i + 4] = (short)(j + 3);
			indices[i + 5] = (short)(j + 0);
		}
		vertices.setIndices(indices, 0, indices.length);
	}

	public void beginBatch(Texture texture) {
		texture.bind();
		bufferIndex = 0;
		numSprites = 0;
	}

	public void endBatch() {
		vertices.setVertices(vertivesBuffer, 0, bufferIndex);
		vertices.bind();
		vertices.draw(GL10.GL_TRIANGLES, 0, numSprites * 6);
		vertices.unbind();
	}

	public void drawSprite(float x, float y, float width, float height,
			TextureRegion region) {
		float halfWidth = width / 2;
		float halfHeight = height / 2;
		float x1 = x - halfWidth;
		float y1 = y - halfHeight;
		float x2 = x + halfWidth;
		float y2 = y + halfHeight;

		vertivesBuffer[bufferIndex++] = x1;
		vertivesBuffer[bufferIndex++] = y1;
		vertivesBuffer[bufferIndex++] = region.u1;
		vertivesBuffer[bufferIndex++] = region.v2;

		vertivesBuffer[bufferIndex++] = x2;
		vertivesBuffer[bufferIndex++] = y1;
		vertivesBuffer[bufferIndex++] = region.u2;
		vertivesBuffer[bufferIndex++] = region.v2;

		vertivesBuffer[bufferIndex++] = x2;
		vertivesBuffer[bufferIndex++] = y2;
		vertivesBuffer[bufferIndex++] = region.u2;
		vertivesBuffer[bufferIndex++] = region.v1;

		vertivesBuffer[bufferIndex++] = x1;
		vertivesBuffer[bufferIndex++] = y2;
		vertivesBuffer[bufferIndex++] = region.u1;
		vertivesBuffer[bufferIndex++] = region.v1;

		numSprites++;
	}

	public void drawSprite(float x, float y, float width, float height, float angle,
			TextureRegion region) {
		float halfWidth = width / 2;
		float halfHeight = height / 2;

		float rad = angle * Vector2.TO_RADIANS;
		float cos = FloatMath.cos(rad);
		float sin = FloatMath.sin(rad);

		float x1 = x + (-halfWidth) * cos - (-halfHeight) * sin;
		float y1 = y + (-halfWidth) * sin + (-halfHeight) * cos;
		float x2 = x + halfWidth * cos - (-halfHeight) * sin;
		float y2 = y + halfWidth * sin + (-halfHeight) * cos;
		float x3 = x + halfWidth * cos - halfHeight * sin;
		float y3 = y + halfWidth * sin + halfHeight * cos;
		float x4 = x + (-halfWidth) * cos - halfHeight * sin;
		float y4 = y + (-halfWidth) * sin + halfHeight * cos;

		vertivesBuffer[bufferIndex++] = x1;
		vertivesBuffer[bufferIndex++] = y1;
		vertivesBuffer[bufferIndex++] = region.u1;
		vertivesBuffer[bufferIndex++] = region.v2;

		vertivesBuffer[bufferIndex++] = x2;
		vertivesBuffer[bufferIndex++] = y2;
		vertivesBuffer[bufferIndex++] = region.u2;
		vertivesBuffer[bufferIndex++] = region.v2;

		vertivesBuffer[bufferIndex++] = x3;
		vertivesBuffer[bufferIndex++] = y3;
		vertivesBuffer[bufferIndex++] = region.u2;
		vertivesBuffer[bufferIndex++] = region.v1;

		vertivesBuffer[bufferIndex++] = x4;
		vertivesBuffer[bufferIndex++] = y4;
		vertivesBuffer[bufferIndex++] = region.u1;
		vertivesBuffer[bufferIndex++] = region.v1;

		numSprites++;
	}
}
